IANA_TLD_TXT='''
# Version 2016081600, Last Updated Tue Aug 16 07:07:01 2016 UTC
AAA
AARP
ABARTH
ABB
ABBOTT
ABBVIE
ABC
ABLE
ABOGADO
ABUDHABI
AC
ACADEMY
ACCENTURE
ACCOUNTANT
ACCOUNTANTS
ACO
ACTIVE
ACTOR
AD
ADAC
ADS
ADULT
AE
AEG
AERO
AETNA
AF
AFAMILYCOMPANY
AFL
AG
AGAKHAN
AGENCY
AI
AIG
AIRBUS
AIRFORCE
AIRTEL
AKDN
AL
ALFAROMEO
ALIBABA
ALIPAY
ALLFINANZ
ALLSTATE
ALLY
ALSACE
ALSTOM
AM
AMERICANEXPRESS
AMERICANFAMILY
AMEX
AMFAM
AMICA
AMSTERDAM
ANALYTICS
ANDROID
ANQUAN
ANZ
AO
APARTMENTS
APP
APPLE
AQ
AQUARELLE
AR
ARAMCO
ARCHI
ARMY
ARPA
ART
ARTE
AS
ASDA
ASIA
ASSOCIATES
AT
ATHLETA
ATTORNEY
AU
AUCTION
AUDI
AUDIBLE
AUDIO
AUTHOR
AUTO
AUTOS
AVIANCA
AW
AWS
AX
AXA
AZ
AZURE
BA
BABY
BAIDU
BANAMEX
BANANAREPUBLIC
BAND
BANK
BAR
BARCELONA
BARCLAYCARD
BARCLAYS
BAREFOOT
BARGAINS
BAUHAUS
BAYERN
BB
BBC
BBT
BBVA
BCG
BCN
BD
BE
BEATS
BEAUTY
BEER
BENTLEY
BERLIN
BEST
BESTBUY
BET
BF
BG
BH
BHARTI
BI
BIBLE
BID
BIKE
BING
BINGO
BIO
BIZ
BJ
BLACK
BLACKFRIDAY
BLANCO
BLOCKBUSTER
BLOG
BLOOMBERG
BLUE
BM
BMS
BMW
BN
BNL
BNPPARIBAS
BO
BOATS
BOEHRINGER
BOFA
BOM
BOND
BOO
BOOK
BOOKING
BOOTS
BOSCH
BOSTIK
BOT
BOUTIQUE
BR
BRADESCO
BRIDGESTONE
BROADWAY
BROKER
BROTHER
BRUSSELS
BS
BT
BUDAPEST
BUGATTI
BUILD
BUILDERS
BUSINESS
BUY
BUZZ
BV
BW
BY
BZ
BZH
CA
CAB
CAFE
CAL
CALL
CALVINKLEIN
CAM
CAMERA
CAMP
CANCERRESEARCH
CANON
CAPETOWN
CAPITAL
CAPITALONE
CAR
CARAVAN
CARDS
CARE
CAREER
CAREERS
CARS
CARTIER
CASA
CASH
CASINO
CAT
CATERING
CBA
CBN
CBRE
CBS
CC
CD
CEB
CENTER
CEO
CERN
CF
CFA
CFD
CG
CH
CHANEL
CHANNEL
CHASE
CHAT
CHEAP
CHINTAI
CHLOE
CHRISTMAS
CHROME
CHRYSLER
CHURCH
CI
CIPRIANI
CIRCLE
CISCO
CITADEL
CITI
CITIC
CITY
CITYEATS
CK
CL
CLAIMS
CLEANING
CLICK
CLINIC
CLINIQUE
CLOTHING
CLOUD
CLUB
CLUBMED
CM
CN
CO
COACH
CODES
COFFEE
COLLEGE
COLOGNE
COM
COMCAST
COMMBANK
COMMUNITY
COMPANY
COMPARE
COMPUTER
COMSEC
CONDOS
CONSTRUCTION
CONSULTING
CONTACT
CONTRACTORS
COOKING
COOKINGCHANNEL
COOL
COOP
CORSICA
COUNTRY
COUPON
COUPONS
COURSES
CR
CREDIT
CREDITCARD
CREDITUNION
CRICKET
CROWN
CRS
CRUISES
CSC
CU
CUISINELLA
CV
CW
CX
CY
CYMRU
CYOU
CZ
DABUR
DAD
DANCE
DATE
DATING
DATSUN
DAY
DCLK
DDS
DE
DEAL
DEALER
DEALS
DEGREE
DELIVERY
DELL
DELOITTE
DELTA
DEMOCRAT
DENTAL
DENTIST
DESI
DESIGN
DEV
DHL
DIAMONDS
DIET
DIGITAL
DIRECT
DIRECTORY
DISCOUNT
DISCOVER
DISH
DJ
DK
DM
DNP
DO
DOCS
DOCTOR
DODGE
DOG
DOHA
DOMAINS
DOT
DOWNLOAD
DRIVE
DTV
DUBAI
DUCK
DUNLOP
DUNS
DUPONT
DURBAN
DVAG
DZ
EARTH
EAT
EC
EDEKA
EDU
EDUCATION
EE
EG
EMAIL
EMERCK
ENERGY
ENGINEER
ENGINEERING
ENTERPRISES
EPOST
EPSON
EQUIPMENT
ER
ERICSSON
ERNI
ES
ESQ
ESTATE
ESURANCE
ET
EU
EUROVISION
EUS
EVENTS
EVERBANK
EXCHANGE
EXPERT
EXPOSED
EXPRESS
EXTRASPACE
FAGE
FAIL
FAIRWINDS
FAITH
FAMILY
FAN
FANS
FARM
FARMERS
FASHION
FAST
FEDEX
FEEDBACK
FERRARI
FERRERO
FI
FIAT
FIDELITY
FILM
FINAL
FINANCE
FINANCIAL
FIRE
FIRESTONE
FIRMDALE
FISH
FISHING
FIT
FITNESS
FJ
FK
FLICKR
FLIGHTS
FLIR
FLORIST
FLOWERS
FLY
FM
FO
FOO
FOODNETWORK
FOOTBALL
FORD
FOREX
FORSALE
FORUM
FOUNDATION
FOX
FR
FRESENIUS
FRL
FROGANS
FRONTDOOR
FRONTIER
FTR
FUJITSU
FUJIXEROX
FUND
FURNITURE
FUTBOL
FYI
GA
GAL
GALLERY
GALLO
GALLUP
GAME
GAMES
GAP
GARDEN
GB
GBIZ
GD
GDN
GE
GEA
GENT
GENTING
GF
GG
GGEE
GH
GI
GIFT
GIFTS
GIVES
GIVING
GL
GLADE
GLASS
GLE
GLOBAL
GLOBO
GM
GMAIL
GMBH
GMO
GMX
GN
GODADDY
GOLD
GOLDPOINT
GOLF
GOO
GOODHANDS
GOODYEAR
GOOG
GOOGLE
GOP
GOT
GOV
GP
GQ
GR
GRAINGER
GRAPHICS
GRATIS
GREEN
GRIPE
GROUP
GS
GT
GU
GUARDIAN
GUCCI
GUGE
GUIDE
GUITARS
GURU
GW
GY
HAMBURG
HANGOUT
HAUS
HBO
HDFCBANK
HEALTH
HEALTHCARE
HELP
HELSINKI
HERE
HERMES
HGTV
HIPHOP
HISAMITSU
HITACHI
HIV
HK
HKT
HM
HN
HOCKEY
HOLDINGS
HOLIDAY
HOMEDEPOT
HOMEGOODS
HOMES
HOMESENSE
HONDA
HONEYWELL
HORSE
HOST
HOSTING
HOT
HOTELES
HOTMAIL
HOUSE
HOW
HR
HSBC
HT
HTC
HU
HUGHES
HYATT
HYUNDAI
IBM
ICBC
ICE
ICU
ID
IE
IEEE
IFM
IINET
IKANO
IL
IM
IMAMAT
IMDB
IMMO
IMMOBILIEN
IN
INDUSTRIES
INFINITI
INFO
ING
INK
INSTITUTE
INSURANCE
INSURE
INT
INTEL
INTERNATIONAL
INTUIT
INVESTMENTS
IO
IPIRANGA
IQ
IR
IRISH
IS
ISELECT
ISMAILI
IST
ISTANBUL
IT
ITAU
ITV
IWC
JAGUAR
JAVA
JCB
JCP
JE
JEEP
JETZT
JEWELRY
JLC
JLL
JM
JMP
JNJ
JO
JOBS
JOBURG
JOT
JOY
JP
JPMORGAN
JPRS
JUEGOS
JUNIPER
KAUFEN
KDDI
KE
KERRYHOTELS
KERRYLOGISTICS
KERRYPROPERTIES
KFH
KG
KH
KI
KIA
KIM
KINDER
KINDLE
KITCHEN
KIWI
KM
KN
KOELN
KOMATSU
KOSHER
KP
KPMG
KPN
KR
KRD
KRED
KUOKGROUP
KW
KY
KYOTO
KZ
LA
LACAIXA
LADBROKES
LAMBORGHINI
LAMER
LANCASTER
LANCIA
LANCOME
LAND
LANDROVER
LANXESS
LASALLE
LAT
LATINO
LATROBE
LAW
LAWYER
LB
LC
LDS
LEASE
LECLERC
LEFRAK
LEGAL
LEGO
LEXUS
LGBT
LI
LIAISON
LIDL
LIFE
LIFEINSURANCE
LIFESTYLE
LIGHTING
LIKE
LILLY
LIMITED
LIMO
LINCOLN
LINDE
LINK
LIPSY
LIVE
LIVING
LIXIL
LK
LOAN
LOANS
LOCKER
LOCUS
LOFT
LOL
LONDON
LOTTE
LOTTO
LOVE
LPL
LPLFINANCIAL
LR
LS
LT
LTD
LTDA
LU
LUNDBECK
LUPIN
LUXE
LUXURY
LV
LY
MA
MACYS
MADRID
MAIF
MAISON
MAKEUP
MAN
MANAGEMENT
MANGO
MARKET
MARKETING
MARKETS
MARRIOTT
MARSHALLS
MASERATI
MATTEL
MBA
MC
MCD
MCDONALDS
MCKINSEY
MD
ME
MED
MEDIA
MEET
MELBOURNE
MEME
MEMORIAL
MEN
MENU
MEO
METLIFE
MG
MH
MIAMI
MICROSOFT
MIL
MINI
MINT
MIT
MITSUBISHI
MK
ML
MLB
MLS
MM
MMA
MN
MO
MOBI
MOBILY
MODA
MOE
MOI
MOM
MONASH
MONEY
MONTBLANC
MOPAR
MORMON
MORTGAGE
MOSCOW
MOTORCYCLES
MOV
MOVIE
MOVISTAR
MP
MQ
MR
MS
MSD
MT
MTN
MTPC
MTR
MU
MUSEUM
MUTUAL
MUTUELLE
MV
MW
MX
MY
MZ
NA
NADEX
NAGOYA
NAME
NATIONWIDE
NATURA
NAVY
NBA
NC
NE
NEC
NET
NETBANK
NETFLIX
NETWORK
NEUSTAR
NEW
NEWS
NEXT
NEXTDIRECT
NEXUS
NF
NFL
NG
NGO
NHK
NI
NICO
NIKE
NIKON
NINJA
NISSAN
NISSAY
NL
NO
NOKIA
NORTHWESTERNMUTUAL
NORTON
NOW
NOWRUZ
NOWTV
NP
NR
NRA
NRW
NTT
NU
NYC
NZ
OBI
OFF
OFFICE
OKINAWA
OLAYAN
OLAYANGROUP
OLDNAVY
OLLO
OM
OMEGA
ONE
ONG
ONL
ONLINE
ONYOURSIDE
OOO
OPEN
ORACLE
ORANGE
ORG
ORGANIC
ORIENTEXPRESS
ORIGINS
OSAKA
OTSUKA
OTT
OVH
PA
PAGE
PAMPEREDCHEF
PANASONIC
PANERAI
PARIS
PARS
PARTNERS
PARTS
PARTY
PASSAGENS
PAY
PCCW
PE
PET
PF
PFIZER
PG
PH
PHARMACY
PHILIPS
PHOTO
PHOTOGRAPHY
PHOTOS
PHYSIO
PIAGET
PICS
PICTET
PICTURES
PID
PIN
PING
PINK
PIONEER
PIZZA
PK
PL
PLACE
PLAY
PLAYSTATION
PLUMBING
PLUS
PM
PN
PNC
POHL
POKER
POLITIE
PORN
POST
PR
PRAMERICA
PRAXI
PRESS
PRIME
PRO
PROD
PRODUCTIONS
PROF
PROGRESSIVE
PROMO
PROPERTIES
PROPERTY
PROTECTION
PRU
PRUDENTIAL
PS
PT
PUB
PW
PWC
PY
QA
QPON
QUEBEC
QUEST
QVC
RACING
RAID
RE
READ
REALESTATE
REALTOR
REALTY
RECIPES
RED
REDSTONE
REDUMBRELLA
REHAB
REISE
REISEN
REIT
REN
RENT
RENTALS
REPAIR
REPORT
REPUBLICAN
REST
RESTAURANT
REVIEW
REVIEWS
REXROTH
RICH
RICHARDLI
RICOH
RIGHTATHOME
RIO
RIP
RO
ROCHER
ROCKS
RODEO
ROOM
RS
RSVP
RU
RUHR
RUN
RW
RWE
RYUKYU
SA
SAARLAND
SAFE
SAFETY
SAKURA
SALE
SALON
SAMSUNG
SANDVIK
SANDVIKCOROMANT
SANOFI
SAP
SAPO
SARL
SAS
SAVE
SAXO
SB
SBI
SBS
SC
SCA
SCB
SCHAEFFLER
SCHMIDT
SCHOLARSHIPS
SCHOOL
SCHULE
SCHWARZ
SCIENCE
SCJOHNSON
SCOR
SCOT
SD
SE
SEAT
SECURE
SECURITY
SEEK
SELECT
SENER
SERVICES
SES
SEVEN
SEW
SEX
SEXY
SFR
SG
SH
SHANGRILA
SHARP
SHAW
SHELL
SHIA
SHIKSHA
SHOES
SHOP
SHOPPING
SHOUJI
SHOW
SHOWTIME
SHRIRAM
SI
SILK
SINA
SINGLES
SITE
SJ
SK
SKI
SKIN
SKY
SKYPE
SL
SLING
SM
SMART
SMILE
SN
SNCF
SO
SOCCER
SOCIAL
SOFTBANK
SOFTWARE
SOHU
SOLAR
SOLUTIONS
SONG
SONY
SOY
SPACE
SPIEGEL
SPOT
SPREADBETTING
SR
SRL
SRT
ST
STADA
STAPLES
STAR
STARHUB
STATEBANK
STATEFARM
STATOIL
STC
STCGROUP
STOCKHOLM
STORAGE
STORE
STREAM
STUDIO
STUDY
STYLE
SU
SUCKS
SUPPLIES
SUPPLY
SUPPORT
SURF
SURGERY
SUZUKI
SV
SWATCH
SWIFTCOVER
SWISS
SX
SY
SYDNEY
SYMANTEC
SYSTEMS
SZ
TAB
TAIPEI
TALK
TAOBAO
TARGET
TATAMOTORS
TATAR
TATTOO
TAX
TAXI
TC
TCI
TD
TDK
TEAM
TECH
TECHNOLOGY
TEL
TELECITY
TELEFONICA
TEMASEK
TENNIS
TEVA
TF
TG
TH
THD
THEATER
THEATRE
TIAA
TICKETS
TIENDA
TIFFANY
TIPS
TIRES
TIROL
TJ
TJMAXX
TJX
TK
TKMAXX
TL
TM
TMALL
TN
TO
TODAY
TOKYO
TOOLS
TOP
TORAY
TOSHIBA
TOTAL
TOURS
TOWN
TOYOTA
TOYS
TR
TRADE
TRADING
TRAINING
TRAVEL
TRAVELCHANNEL
TRAVELERS
TRAVELERSINSURANCE
TRUST
TRV
TT
TUBE
TUI
TUNES
TUSHU
TV
TVS
TW
TZ
UA
UBS
UCONNECT
UG
UK
UNICOM
UNIVERSITY
UNO
UOL
UPS
US
UY
UZ
VA
VACATIONS
VANA
VC
VE
VEGAS
VENTURES
VERISIGN
VERSICHERUNG
VET
VG
VI
VIAJES
VIDEO
VIG
VIKING
VILLAS
VIN
VIP
VIRGIN
VISA
VISION
VISTA
VISTAPRINT
VIVA
VIVO
VLAANDEREN
VN
VODKA
VOLKSWAGEN
VOTE
VOTING
VOTO
VOYAGE
VU
VUELOS
WALES
WALTER
WANG
WANGGOU
WARMAN
WATCH
WATCHES
WEATHER
WEATHERCHANNEL
WEBCAM
WEBER
WEBSITE
WED
WEDDING
WEIBO
WEIR
WF
WHOSWHO
WIEN
WIKI
WILLIAMHILL
WIN
WINDOWS
WINE
WINNERS
WME
WOLTERSKLUWER
WOODSIDE
WORK
WORKS
WORLD
WS
WTC
WTF
XBOX
XEROX
XFINITY
XIHUAN
XIN
XN--11B4C3D
XN--1CK2E1B
XN--1QQW23A
XN--30RR7Y
XN--3BST00M
XN--3DS443G
XN--3E0B707E
XN--3PXU8K
XN--42C2D9A
XN--45BRJ9C
XN--45Q11C
XN--4GBRIM
XN--55QW42G
XN--55QX5D
XN--5SU34J936BGSG
XN--5TZM5G
XN--6FRZ82G
XN--6QQ986B3XL
XN--80ADXHKS
XN--80AO21A
XN--80ASEHDB
XN--80ASWG
XN--8Y0A063A
XN--90A3AC
XN--90AE
XN--90AIS
XN--9DBQ2A
XN--9ET52U
XN--9KRT00A
XN--B4W605FERD
XN--BCK1B9A5DRE4C
XN--C1AVG
XN--C2BR7G
XN--CCK2B3B
XN--CG4BKI
XN--CLCHC0EA0B2G2A9GCD
XN--CZR694B
XN--CZRS0T
XN--CZRU2D
XN--D1ACJ3B
XN--D1ALF
XN--E1A4C
XN--ECKVDTC9D
XN--EFVY88H
XN--ESTV75G
XN--FCT429K
XN--FHBEI
XN--FIQ228C5HS
XN--FIQ64B
XN--FIQS8S
XN--FIQZ9S
XN--FJQ720A
XN--FLW351E
XN--FPCRJ9C3D
XN--FZC2C9E2C
XN--FZYS8D69UVGM
XN--G2XX48C
XN--GCKR3F0F
XN--GECRJ9C
XN--H2BRJ9C
XN--HXT814E
XN--I1B6B1A6A2E
XN--IMR513N
XN--IO0A7I
XN--J1AEF
XN--J1AMH
XN--J6W193G
XN--JLQ61U9W7B
XN--JVR189M
XN--KCRX77D1X4A
XN--KPRW13D
XN--KPRY57D
XN--KPU716F
XN--KPUT3I
XN--L1ACC
XN--LGBBAT1AD8J
XN--MGB9AWBF
XN--MGBA3A3EJT
XN--MGBA3A4F16A
XN--MGBA7C0BBN0A
XN--MGBAAM7A8H
XN--MGBAB2BD
XN--MGBAYH7GPA
XN--MGBB9FBPOB
XN--MGBBH1A71E
XN--MGBC0A9AZCG
XN--MGBCA7DZDO
XN--MGBERP4A5D4AR
XN--MGBPL2FH
XN--MGBT3DHD
XN--MGBTX2B
XN--MGBX4CD0AB
XN--MIX891F
XN--MK1BU44C
XN--MXTQ1M
XN--NGBC5AZD
XN--NGBE9E0A
XN--NODE
XN--NQV7F
XN--NQV7FS00EMA
XN--NYQY26A
XN--O3CW4H
XN--OGBPF8FL
XN--P1ACF
XN--P1AI
XN--PBT977C
XN--PGBS0DH
XN--PSSY2U
XN--Q9JYB4C
XN--QCKA1PMC
XN--QXAM
XN--RHQV96G
XN--ROVU88B
XN--S9BRJ9C
XN--SES554G
XN--T60B56A
XN--TCKWE
XN--UNUP4Y
XN--VERMGENSBERATER-CTB
XN--VERMGENSBERATUNG-PWB
XN--VHQUV
XN--VUQ861B
XN--W4R85EL8FHU5DNRA
XN--W4RS40L
XN--WGBH1C
XN--WGBL6A
XN--XHQ521B
XN--XKC2AL3HYE2A
XN--XKC2DL3A5EE0H
XN--Y9A3AQ
XN--YFRO4I67O
XN--YGBI2AMMX
XN--ZFR164B
XPERIA
XXX
XYZ
YACHTS
YAHOO
YAMAXUN
YANDEX
YE
YODOBASHI
YOGA
YOKOHAMA
YOU
YOUTUBE
YT
YUN
ZA
ZAPPOS
ZARA
ZERO
ZIP
ZIPPO
ZM
ZONE
ZUERICH
ZW
'''

IANA_TLD_LIST = []
for line in IANA_TLD_TXT.split('\n'):
	if len(line)==0:
		continue
	if line[0]=='#':
		continue
	IANA_TLD_LIST.append(line.lower())




import re
import logging

from .ianatlds import IANA_TLD_LIST

class StringFinder:

	re_dns = re.compile('([a-zA-Z0-9]{1}[-.a-zA-Z0-9]+)')
	re_windowend = re.compile('.*?([-.a-zA-Z0-9]+)$')

	def __init__(self, tlds=None, onlySuffixes=None):
		self.onlySuffixes=onlySuffixes
		if tlds!=None:
			self.tlds = tlds
		else:
			self.tlds = set(IANA_TLD_LIST)
		
		self.log = logging.getLogger('nmine.strfind')

	# called from searchString
	def divideString(self, s): 
	
		"""takes a string and returns a tuple of 3 strings (not, maybe,
		rest). not+maybe+rest = s. Up to two of (not,maybe,rest) can be ''. 
		not is text at the start of s which can't be a DNS name. maybe is a
		string in s which could be a dns name. rest is a substring after
		maybe which may or may not be a dns name (pass to divideString again
		to find out)."""
		
		mg = self.re_dns.search(s)
		if mg:
			notdns = s[0:mg.start()]
			maybedns = mg.group(1)
			rest = s[mg.end():]
			
			return (notdns, maybedns, rest)
		else:
			return (s, '', '')
	
	# called from searchStringConfirmed
	def searchString(self, s):
	
		"""return substrings in s that are could possibly be DNS names"""
	
		names = set()
		while True:
			(notdns, maybedns, rest) = self.divideString(s)
			
			if maybedns!='':
				names.add(maybedns)
			
			if rest=='':
				return names
			
			s = rest

	# MAIN ENTRY POINT
	def searchStringConfirmed(self, s):
		
		"""does the same as searchString, but only returns the subset of
		strings which have known TLD endings."""
		
		names = self.searchString(s)
		namesConfirmed = set()
		for name in names:
			if name.endswith('.'):
				name=name[:-1]
			
			name = name.lower()
			
			parts = name.split('.')
			if len(parts)>1 and parts[-1] in self.tlds and self.nameMatchesOnlySuffixes(name):
				namesConfirmed.add(name)

		return namesConfirmed

	def nameMatchesOnlySuffixes(self, name):
		if self.onlySuffixes==None:
			return True
		else:
			for suffix in self.onlySuffixes:
				if name.endswith(suffix.lower()):
					return True
			return False
						




import re
from .ianatlds import IANA_TLD_LIST

class StreamFinder:
	
	endings = ['.'+tld for tld in IANA_TLD_LIST]
	re_dns = re.compile('([-.a-zA-Z0-9]+)')
	re_windowend = re.compile('.*?([-.a-zA-Z0-9]+)$')

	blacklist = ('document.do','asp.net')

	def __init__(self):
		self.pos=0
		self._buf = ''
		self.extraTLDprovider = lambda: set()

	def updateTLDs(self):
		print('NEW TLDs')
		pass

	@classmethod
	def tokenizeString(cls, s):
		sf = cls()
		sf.feed(s)
		print('wsr:',cls.searchWindow(s))
		#return s

		"""exhaustively check self._buf until there's no hope for finding
		any DNS names in it."""
		
		#assert False
		rv=[]
		
		while True:
			#print 'buffer check round:',repr(self._buf)
			(name, newbuf) = cls.searchWindow(self._buf)
			print('got name=%s newbuf=%s' % (repr(name),repr(newbuf)))
			if name!=None:
				#yield name
				rv += {'type':'dns', 'value':name}
			elif name==None:
				pass
				
			if newbuf==self._buf:
				return
			self._buf = newbuf

	@classmethod	
	def searchWindow(cls, s):
	
		"""search for a DNS name in the buffer. If one is found, return the
		DNS name and the REST of the input string. If one is not found,
		return any ending of the input string the could possibly be the
		start of a DNS name if more data were appended."""
	
		#for i in range(0,len(self._buf)):
		mg = cls.re_dns.search(s)
		if mg:
			start = mg.start()
			end = mg.end()
			#word = self._buf[start:end]
			word = mg.group(1)
			
			#print 'possibility:',repr(word)
			for ending in cls.endings or ending in ['.'+tld for tld in self.extraTLDprovider]:
				if word.lower().endswith(ending) and word.lower()!=ending and word.lower() not in cls.blacklist:
					#print 'NAME:',word
					return (s[0:start], word, s[end:])
			
			# if we had a possibility but it isn't real, AND it goes
			# all the way to the end of the string, then preserve the ending
			if end==len(s):
				return ('',None, s)
			else:	
				return (s[0:start], None, s[end:])
		else:
			mg = cls.re_windowend.match(s)			
			if mg:
				#return (s[0:len(mg.group(1)], None, mg.group(1))
				pass
			else:
				#print 'window end match failure:',repr(s)
				return (s, None, '')
	
	def checkBuffer(self):

		"""exhaustively check self._buf until there's no hope for finding
		any DNS names in it."""
		
		while True:
			###Bprint 'buffer check round:',repr(self._buf)
			(start, name, newbuf) = self.__class__.searchWindow(self._buf)
			###print 'got name=%s newbuf=%s' % (repr(name),repr(newbuf))
			if name:
				yield name
			if newbuf==self._buf:
				return
			self._buf = newbuf
	
	def feed(self, chunk):
		self._buf += chunk
		for name in self.checkBuffer():
			yield name
	
	def searchPath(self, path):
		f=file(path)
	
	@classmethod
	def searchFile(cls, f):
		sf = cls()
		chunk = f.read(1024)
		while chunk:
			for name in sf.feed(chunk):
				yield name
			chunk = f.read(1024)
	
	@classmethod
	def searchPath(cls, path):
		f=file(path)
		return cls.searchFile(f)
	
	@classmethod
	def searchDir(cls, path):
		for (dirpath, drnames, filenames) in os.walk(path):
			for filename in filenames:
				path = os.path.join(dirpath,filename)
				#print 'checking',path
				for name in cls.searchPath(path):
					yield name
					
				
				




import sys
import os
import optparse
import logging

from .strfind import StringFinder
from .output import writeOutputZone
from .output import writeOutputHosts
from .ianatlds import IANA_TLD_LIST

def extractNamesFromPath(path, tlds, onlySuffixes=None):
	'''given a path to a file, extract all possible DNS names from it.'''
	log = logging.getLogger('nmine')
	try:
		f = open(path, errors='ignore')
	except FileNotFoundError as fnfe:
		log.error(fnfe)
		return set()
	sf=  StringFinder(tlds=tlds, onlySuffixes=onlySuffixes)
	names = set()
	for line in f:
		for name in sf.searchStringConfirmed(line):
			names.add(name)
	return names		

def parseArgs(argv=sys.argv):
	
	parser = optparse.OptionParser(
		usage="%prog [-i <file>] <path> [<path> ...]",
		version="%prog v1.0"
	)
	parser.remove_option('--version')
	parser.add_option('-i', '--interesting',
		dest='scope',
		default=None,
		help= '''Path to file containing IPv4 addresses or prefixes which you consider "interesting" (the scope of the search).  This program will only output DNS names resolving to these addresses.  If not specified, a file named SCOPE in the current directory will be looked for and used if found.  Either a SCOPE file or this option is required.''')
	parser.add_option('-n', '--nameserver',
		dest='nameserver',
		default=None,
		help= 'Nameserver to query. The normal DNS servers for this system will be used by default.')
	parser.add_option('-f', '--format',
		dest='format',
		default='zone',
		help='Output format. Must be either "hosts" (/etc/hosts style) or "zone" (BIND zone file / dig output style).')
	parser.add_option('-v', '--version',
		action='version',
		help='Display version number and exit')
	parser.add_option('-d', '--debug',
		action='store_true',
		default=False,
		help='Turn on debug logging output' )
	parser.add_option('-t', '--addtld',
		action='append',
		dest='addTlds',
		default=[],
		type='str',
		help='Specify additional TLDs to consider valid. May be specified multiple times.')
	parser.add_option('-T', '--noianatlds',
		action='store_false',
		dest='useIanaTlds',
		default=True,
		help='Do NOT automatically consider IANA TLDs valid. The only TLDs considered valid will be those specified with -t.')
	parser.add_option('-s','--onlysuffix',
		action='append',
		dest='onlySuffixes',
		default=[],
		type='str',
		help='Only consider names with this ending to be valid names at all. May be specified multiple times.')
			
	(opts,args) = parser.parse_args(argv)

	searchItems = []
	if len(args)>=2:
		searchItems = args[1:]
	else:
		searchItems = ['.']

	if opts.format not in ('zone','hosts'):
		sys.stderr.write('Invalid value for -f/--format option. It must be "zone" or "hosts".\n')
		raise SystemExit(1)

	return (searchItems, opts)

def initLogging(debug):
	log = logging.getLogger('nmine')
	handler = logging.StreamHandler(sys.stderr)
	if debug:
		formatter = logging.Formatter('%(asctime)s %(name)s: %(message)s','%X')
	else:
		formatter = logging.Formatter('%(name)s: %(message)s')
	handler.setFormatter(formatter)
	log.addHandler(handler)
	if debug:
		log.setLevel(logging.DEBUG)
	else:
		log.setLevel(logging.ERROR)
	
class ScopeParseError(Exception):
	def __init__(self, scopePath, lineNum, errStr):
		self.scopePath = scopePath
		self.lineNum = lineNum
		self.errStr = errStr
	
	def __str__(self):
		return 'Error reading scope file %s on line %d: %s' % (
			self.scopePath,
			self.lineNum,
			self.errStr
		)
	
def loadScope(scopePath):
	log = logging.getLogger('nmine')

	scope = []

	if scopePath==None:
		scopePath='SCOPE'

	log.debug('loading scope %s' % scopePath)

	import ipcidrtree

	lineNum=0
	for line in open(scopePath):
		lineNum+=1
		
		parts = line.split('#')
		line = parts[0]
		line = line.strip()
		
		if line=='':
			continue
		
		try:
			p = ipcidrtree.Prefix(line)
		except ValueError as ve:
			raise ScopeParseError(scopePath, lineNum, str(ve))
		else:
			scope.append(p)
	
	return scope

class BadSearchPath(Exception):
	def __init__(self, searchPath):
		self.searchPath = searchPath

class NonExistentSearchPath(BadSearchPath):
	def __str__(self):
		return 'search path %s does not exist' % repr(self.searchPath)

class InvalidSearchPath(BadSearchPath):
	def __str__(self):
		return 'search path %s is neither a file nor a directory.' % repr(self.searchPath)

def buildSearchFiles(searchPaths, doDotFiles=False):
	log = logging.getLogger('nmine')
	searchFiles = []
	for searchPath in searchPaths:
		if not os.path.exists(searchPath):
			raise NonExistentSearchPath(searchPath)
		else:
			if os.path.isfile(searchPath):
				log.debug('will search file %s' % searchPath)
				searchFiles.append(searchPath)
			elif os.path.isdir(searchPath):
				log.debug('recursively searching directory %s for files to search' % searchPath)
				for (dirpath, dirnames, filenames) in os.walk(searchPath, topdown=True):
					if not doDotFiles:
						for dirname in dirnames:
							if dirname.startswith('.'):
								del dirnames[dirnames.index(dirname)]
						filenames = [f for f in filenames if not f.startswith('.')]
					for filename in filenames:		
						sf = os.path.join(dirpath, filename)
						log.debug('will search file %s' % sf)
						searchFiles.append(sf)
			else:
				raise InvalidSearchPath(searchPath)
	return searchFiles

def resolveNames(names, scope, nameserver=None):
	import dns.resolver
	import ipcidrtree
	log = logging.getLogger('nmine')
	
	results = {}
	
	if nameserver==None:
		# use the normal system DNS servers
		r = dns.resolver.Resolver(configure=True)
	else:
		# use the specified DNS server, not the system ones
		r = dns.resolver.Resolver(configure=False)
		r.nameservers = [nameserver]

	for name in names:
		log.debug('looking up %s ...' % name)
		try:
			answer = r.query(name,'A')
		except dns.resolver.NoNameservers:
			pass
		except dns.resolver.NXDOMAIN:
			pass
		except dns.resolver.NoAnswer:
			pass
		except dns.exception.Timeout:
			pass
		except dns.name.LabelTooLong:
			pass
		except dns.name.NameTooLong:
			pass
		except dns.name.EmptyLabel:
			# raised if ".." is in a name. parser shouldn't even return that. TODO
			pass
		else:
			for rdata in answer:
				log.debug('resolved %s to %s' % (name, rdata.address))
				prefix = ipcidrtree.Prefix(rdata.address)
				for scopeEntry in scope:
					log.debug('    %s in %s ?' % (prefix, scopeEntry))
					if prefix in scopeEntry or prefix==scopeEntry:
						log.debug('        HIT: %s' % rdata.address)
						if name in results:
							results[name].append(rdata.address)
						else:
							results[name] = [rdata.address]
	return results

def main():
	
	(searchPaths, opts) = parseArgs()
	scopePath = opts.scope
	debug = opts.debug
	initLogging(debug)
	log = logging.getLogger('nmine')

	###################################################
	# make sure required python modules are available in a user-friendly
	# manner
	###################################################

	try:
		import ipcidrtree
	except ImportError:
		log.error("You don't seem to have the ipcidrtree Python module installed. See https://github.com/xelphene/ipcidrtree.")
		raise SystemExit(1)

	try:
		import dns.resolver
	except ImportError:
		log.error("You don't seem to have dnspython installed. See http://www.dnspython.org. (On Ubuntu/Debian: apt-get install python-dnspython)")
		raise SystemExit(1)

	###################################################
	# load the scope
	###################################################

	try:
		scope = loadScope(scopePath)
	except IOError as ioe:
		log.error('Error loading scope file %s: %s. Try the -i <path> command line option.' % (ioe.filename, str(ioe)))
		raise SystemExit(1)
	except ScopeParseError as spe:
		log.error(spe)
		raise SystemExit(1)
	
	if debug:
		for s in scope:
			log.debug('scope item: %s' % s)

	###################################################
	# build up a list of TLDs to consider valid
	###################################################
	
	tlds = []
	if opts.useIanaTlds:
		tlds += IANA_TLD_LIST
	for tld in opts.addTlds:
		tlds.append( tld )
	log.debug('TLD list:')
	for tld in tlds:
		log.debug('    %s' % tld)
	log.debug('END TLD list')
	tlds = set(tlds)

	###################################################
	# build up a list of general suffixes to consider valid
	###################################################
	
	onlySuffixes=None
	if len(opts.onlySuffixes)>0:
		onlySuffixes = opts.onlySuffixes
		log.debug('onlySuffixes enabled: %s' % ', '.join(onlySuffixes))
	else:
		log.debug('onlySuffixes disabled. Any name ending with a known TLD considered valid.')

	###################################################
	# build up a list of paths of plain files to extract possible names from
	###################################################

	log.debug('search paths: %s' % repr(searchPaths))
	try:
		searchFiles = buildSearchFiles(searchPaths)
	except BadSearchPath as bsp:
		log.error(bsp)
		raise SystemExit(1)
	
	###################################################
	# extract possible DNS names from the files discovered above
	###################################################
	
	allNames = set()
	for searchFile in searchFiles:
		log.debug('*** extracting names from %s' % searchFile)
		names = extractNamesFromPath(searchFile, tlds, onlySuffixes)
		for name in names:
			log.debug('    %s found in %s' % (name, searchFile))
		allNames = allNames.union(names)
	log.debug('extraction complete. possible names found:')
	for name in allNames:
		log.debug('    %s' % name)

	###################################################
	# actually resolve the names
	###################################################
	
	lookups = resolveNames(allNames, scope, nameserver=opts.nameserver)
	
	if opts.format=='hosts':
		writeOutputHosts(lookups)
	else:
		writeOutputZone(lookups)




import sys

def writeOutputZone(lookups, outFile=sys.stdout, ttl=600):
	for name in list(lookups.keys()):
		for addr in lookups[name]:
			outFile.write('%-40s %d IN A %s\n' % (
				name+'.', ttl, addr
			))

def writeOutputHosts(lookups, outFile=sys.stdout):
	for name in list(lookups.keys()):
		for addr in lookups[name]:
			outFile.write('%-20s %s\n' % (
				addr, name
			))

